Segment

Struct Segment 

pub struct Segment<T: AsRef<[u8]>, B> { /* private fields */ }
Binary consisting of a single segment of encoded Instructions

This Binary serves a single buffer as a code segment starting from address 0.

Example

The following example builds a segment at a specifig offset:

use riscv_etrace::binary::{self, Adaptable, Binary};
use riscv_etrace::instruction::{self, base};

let bootrom = b"\x97\x02\x00\x00\x93\x85\x02\x02\x73\x25\x40\xf1\x83\xb2\x82\x01\x67\x80\x02\x00";
let mut bootrom = binary::from_segment(bootrom, base::Set::Rv64I)
    .with_offset(0x1000);
assert_eq!(
    bootrom.get_insn(0x1010),
    Ok(instruction::Kind::new_jalr(0, 5, 0).into()),
);

Implementations

Source

impl<T: AsRef<[u8]>, B> Segment<T, B>

pub fn new(data: T, base: B) -> Self

Create a new Binary for code of a given instruction base set