pub enum Error {
NoSegmentFound,
CouldNotRetrieveData(ParseError),
ExceededHostUSize(TryFromIntError),
InvalidInstruction,
UnsupportedArchitecture,
UnsupportedEndianess,
}Available on crate feature
elf only.Expand description
ELF specific error type
Variants
NoSegmentFound
No segment was found containing the address
CouldNotRetrieveData(ParseError)
The data for a segment could not be retrieved
ExceededHostUSize(TryFromIntError)
Could not use an address or offset because it is too big for the host
InvalidInstruction
An Instruction could not be extracted from the data
UnsupportedArchitecture
The ELF file is not an RV32 ELF file
UnsupportedEndianess
The ELF file is not little endian