Encoder

Struct Encoder 

pub struct Encoder<'d, U> { /* private fields */ }
Am encoder for packets and/or payloads

Implementations

Source

impl<'d, U> Encoder<'d, U>

pub fn reset(&mut self, data: &'d mut [u8])

Reset the inner data to the given byte slice

pub fn uncommitted(&self) -> usize

Retrieve the number of bytes in the buffer that are not committed

pub fn encode(&mut self, data: &impl Encode<'d, U>) -> Result<(), Error>

Encode one entity

pub fn unit(&self) -> &U

Retrieve this encoder’s unit